/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_avr2_emulation;
extern ld_emulation_xfer_type ld_avr1_emulation;
extern ld_emulation_xfer_type ld_avr25_emulation;
extern ld_emulation_xfer_type ld_avr3_emulation;
extern ld_emulation_xfer_type ld_avr31_emulation;
extern ld_emulation_xfer_type ld_avr35_emulation;
extern ld_emulation_xfer_type ld_avr4_emulation;
extern ld_emulation_xfer_type ld_avr5_emulation;
extern ld_emulation_xfer_type ld_avr51_emulation;
extern ld_emulation_xfer_type ld_avr6_emulation;
extern ld_emulation_xfer_type ld_avrxmega1_emulation;
extern ld_emulation_xfer_type ld_avrxmega2_emulation;
extern ld_emulation_xfer_type ld_avrxmega2_flmap_emulation;
extern ld_emulation_xfer_type ld_avrxmega3_emulation;
extern ld_emulation_xfer_type ld_avrxmega4_emulation;
extern ld_emulation_xfer_type ld_avrxmega4_flmap_emulation;
extern ld_emulation_xfer_type ld_avrxmega5_emulation;
extern ld_emulation_xfer_type ld_avrxmega6_emulation;
extern ld_emulation_xfer_type ld_avrxmega7_emulation;
extern ld_emulation_xfer_type ld_avrtiny_emulation;

#define EMULATION_LIST \
  &ld_avr2_emulation, \
  &ld_avr1_emulation, \
  &ld_avr25_emulation, \
  &ld_avr3_emulation, \
  &ld_avr31_emulation, \
  &ld_avr35_emulation, \
  &ld_avr4_emulation, \
  &ld_avr5_emulation, \
  &ld_avr51_emulation, \
  &ld_avr6_emulation, \
  &ld_avrxmega1_emulation, \
  &ld_avrxmega2_emulation, \
  &ld_avrxmega2_flmap_emulation, \
  &ld_avrxmega3_emulation, \
  &ld_avrxmega4_emulation, \
  &ld_avrxmega4_flmap_emulation, \
  &ld_avrxmega5_emulation, \
  &ld_avrxmega6_emulation, \
  &ld_avrxmega7_emulation, \
  &ld_avrtiny_emulation, \
  0
